/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#ifndef DOORBELL_H_
#define DOORBELL_H_

#include "mmstools/mmstimer.h"
#include "mainmenu.h"
#include "pindialog.h"
#include "errordialog.h"
#include "absencedialog.h"
#include "ringingdialog.h"
#include "volumedialog.h"
#include "sipbackend_minisip.h"
#include "userfilereader.h"
#include "gpiothread.h"

enum DialogType {
	UNKNOWN_DIALOG,
	PIN_DIALOG,
	ERROR_DIALOG,
	ABSENCE_DIALOG,
	RINGING_DIALOG,
	VOLUME_DIALOG
};

struct CleanUpPtr {
		DialogType type;
		Dialog *dialog;
};

class DoorBell {
	public:
		DoorBell();
		~DoorBell();

		void start();

	private:
		std::string    pin;
		std::string    errorMsg;
		std::string    timerMsg;
		int            timerLength;
		MMSConfigData  config;
		USER_MAP       users;
		MainMenu      *mainMenu;
		Dialog        *currentDialog;
		PinDialog     *pinDialog;
		ErrorDialog   *errorDialog;
		AbsenceDialog *absenceDialog;
		RingingDialog *ringingDialog;
		VolumeDialog  *volumeDialog;
		MMSTimer      *showMenuTimer;

		Sipbackend 		*sipController;

#ifdef __EPCBF527__
		GPIOThread		*gpioThread;
#endif

		void onSelectItemMainMenu(MMSWidget *wid);
		void onLodgerSelect(MMSWidget *wid);
		void onPinSelect(MMSWidget *wid);
		void showErrorDialog(std::string errorMsg, bool isCritical = false,
				std::string timerMsg = "", int length = 0);

		void onHideDialog(Dialog *dialog, bool showMainWindow = true);
		void onReturnPinDialog(MMSWidget *widget);
		void onTimeOut();
		void onError(std::string errorMsg, bool isCritical,
				std::string timerMsg, int length);

		
		void onEstablished(void *);
		void onDisconnect(void *);
		void onRegister(RegisterStatus);
		
		void sendErrorMail(std::string &errMsg);

		bool inCall();
};

#endif /* DOORBELL_H_ */
