/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#ifndef SIPBACKEND_H_
#define SIPBACKEND_H_

#include <iostream>

#include <sigc++/sigc++.h>

#include <mmstools/mmserror.h>

#ifdef __HAVE_MIXER__
#include <mmsmedia/mmsaudioctrl.h>
#endif

enum RegisterStatus {
	TRYING,
	FAILED,
	OK
};

class Sipbackend {
	public:
		Sipbackend();
		virtual ~Sipbackend() {};

		virtual bool initialize() = 0;
		virtual bool shutdown() = 0;

		virtual bool call(const std::string &user, const std::string &domain) = 0;
		virtual bool hangup() = 0;
		virtual bool answer() = 0;
		virtual bool isInitialized() = 0;
		virtual bool inCall() = 0;
		virtual bool inTrying() = 0;
		virtual bool setVolume(const unsigned int percent);

		sigc::signal<void, void *>            					onEstablished;
		sigc::signal<void, void *>            					onDisconnect;
		sigc::signal<void, std::string, bool, std::string, int> onError;
		sigc::signal<void, RegisterStatus> 						onRegister;

	protected:
		std::string	stunserverIP,
					nameserver,
					interface,
					audioInDevice,
					audioOutDevice,
					ringtone,
					busytone,
					calltone;

		list<std::string> codecs;

		unsigned int volume,
					 stunserverPort;

		struct _account {
			string			user;
			string			passwd;
			string			registrar;
			string			realm;
		} account;

		bool accountRegistered;

#ifdef __HAVE_MIXER__
		MMSAudioCtrl						*audioCtrl;
#endif

		bool readConfig();

	private:
		std::string configFile;
};

MMS_CREATEERROR(MMSSipBackendError);

#endif /* SIPBACKEND_H_ */
