/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#ifdef __USE_MINISIP__

#ifndef SIPBACKEND_MINISIP_H_
#define SIPBACKEND_MINISIP_H_

#include <libminisip/signaling/sip/SipSoftPhoneConfiguration.h>
#include <libminisip/gui/ConsoleDebugger.h>

#include <mmsbase/mmseventsignup.h>
#include <mmstools/mmstimer.h>

#ifndef MINISIP_RINGTONE_SUPPORT
#include "ringtoneThread.h"
#endif

#include "sipbackend.h"

class SipbackendMinisip :  public CommandReceiver, public Sipbackend {
	private:
		string				currentCallID,
							callUser,
							inviteCallID;

		bool				minisipInitialized,
							muted;

		MMSEventSignup      *eventSignup, *eventSignupB;
		MMSTimer			*tryingTimer;

		MRef<SipSoftPhoneConfiguration *>	sipConf;
		MRef<SubsystemMedia *> 				subMedia;
		MRef<Sip *>							sip;
		MRef<SipStack *>					sipStack;

#ifdef __ENABLE_LOG__
		MRef<ConsoleDebugger *>				consoleDbg;
#endif

#ifndef MINISIP_RINGTONE_SUPPORT
		RingtoneThread		*ringtoneThread;
#endif

		bool	initializeMiniSip();
		bool	registerAccounts();
		void	reRegisterAccount();

		void	startRingtone();
		void	startCalltone();
		void	stopTone();

		void	onTryingTimeOut();

	public:
		/* constructor/destructor */
		SipbackendMinisip();
		~SipbackendMinisip();

		bool initialize();
		bool shutdown();

		/* SipDefaultHandler interface implementation */
		void handleCommand(string subsystem, const CommandString &cmd);
		CommandString handleCommandResp(string subsystem, const CommandString &cmd);

		bool    call(const string &user, const string &domain);
		bool 	hangup();
		bool 	answer();
		bool 	isInitialized();
		bool 	inCall();
		bool    inTrying();
};

#endif /*SIPBACKEND_MINISIP_H_*/

#endif /* __USE_MINISIP__ */
