/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#include <cstdio>

#include "dialog.h"

Dialog::Dialog(bool hasCancelBtn) :
		hasCancelBtn(hasCancelBtn),
		exitCode(0),
		win(NULL),
		cancelBtn(NULL) {
}

Dialog::~Dialog() {
}

void Dialog::show() {
	if(win) {
		this->win->show();
		this->win->waitUntilShown();
	}
}

void Dialog::hide() {
	if(win) {
		this->win->hide();
		this->win->waitUntilHidden();
	}

	onExit.emit(this,true);
}

int Dialog::getExitCode() {
	return exitCode;
}

void Dialog::loadDialog(const char *name) {
	try {
		string filename = config.getData() + "/" + name;
		this->win = this->dm.loadDialog(filename,NULL);

		if(this->hasCancelBtn) {
			this->cancelBtn = this->win->searchForWidget("cancelButton");
			this->cancelBtn->onReturn->connect(
					sigc::mem_fun(this, &Dialog::onReturn));
		}

	} catch(MMSDialogManagerError *err) {
		fprintf(stderr, "could not load dialog file %s! Reason %s\n", name, err->getMessage().c_str());
		exit(1);
	}
}

void Dialog::onReturn(MMSWidget *wid) {
	if(wid == this->cancelBtn) {
		this->hide();
	}
}
