/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#ifdef __EPCBF527__

#include "gpiothread.h"
#include <mmsgui/theme/mmsthememanager.h>

extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
}

enum buttonPins {
	BUTTON0 = 80,
	BUTTON1,
	BUTTON2,
	BUTTON3,
	BUTTON4,
	BUTTON5
};

GPIOThread::GPIOThread(Sipbackend *sip) : sip(sip) {}
	
GPIOThread::~GPIOThread() {}

FILE* GPIOThread::initializeButton(unsigned char pin) {
	FILE *fp;
	char tmp[34];

	/* export pin */
	if(!(fp = fopen("/sys/class/gpio/export", "ab"))) {
		DEBUGMSG("GPIO", "error opening file /sys/class/gpio/export");
		throw GPIOError(0, "error opening file /sys/class/gpio/export");
	}

	rewind(fp);
	fprintf(fp, "%2d", pin);
	fclose(fp);

	/* set direction */
	snprintf(tmp, sizeof(tmp), "/sys/class/gpio/gpio%d/direction", pin);
	if(!(fp = fopen(tmp, "rb+"))) {
		DEBUGMSG("GPIO", "error opening file " + string(tmp));
		throw GPIOError(0, "error opening file " + string(tmp));
	}

	rewind(fp);
	fprintf(fp, "in");
	fclose(fp);

	snprintf(tmp, sizeof(tmp), "/sys/class/gpio/gpio%d/value", pin);
	if(!(fp = fopen(tmp, "r"))) {
		DEBUGMSG("GPIO", "error opening file " + string(tmp));
		throw GPIOError(0, "error opening file " + string(tmp));
	}
	
	return fp;
}

void GPIOThread::threadMain() {
	string			theme = "night";
	FILE 			*fp[6];
	unsigned char 	buf;
	bool			pressed[6] = {false, false, false, false, false, false};
	MMSThemeManager tm;
	
	fp[0] = initializeButton(BUTTON0);
	fp[1] = initializeButton(BUTTON1);

	while(true) {
		/* check for button 0 */
		rewind(fp[0]);
		if(fread(&buf, 1, 1, fp[0]) == 1) {
			if(!pressed[0]) {
				if(buf == '0') {
					pressed[0] = true;
					(theme == "night" ? theme = "day" : theme = "night");
					tm.setTheme(theme, true);
				}
			} else if(buf == '1') {
				pressed[0] = false;
			}
		}

		/* check for button 1 */
		rewind(fp[1]);
		if(fread(&buf, 1, 1, fp[1]) == 1) {
			if(!pressed[1]) {
				if(buf == '0') {
					pressed[1] = true;
					this->sip->hangup();
				}
			} else if(buf == '1') {
				pressed[1] = false;
			}
		}

		usleep(50000);
	}

	fclose(fp[0]);
	fclose(fp[1]);
}

#endif /* __EPCBF527__ */
