/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#include "mainmenu.h"
#include <disko.h>

MainMenu::MainMenu(USER_MAP *users) :
		Dialog::Dialog(false),
		lodgerMenu(NULL),
		pinButton(NULL),
		users(users),
		up(NULL),
		down(NULL),
		lastSel(5),
		lastAct(ACT_UKN),
		anim_saved_screen(NULL) {

//	Dialog::Dialog();

	loadDialog("menu.xml");

	//fill the userlist
	MMSWidget *lodger;
	MMSLabelWidget *lodgername;
	MMSCheckBoxWidget *checkbox;

	lodgerMenu = dynamic_cast<MMSMenuWidget *>(this->win->searchForWidget("namesMenu"));
	pinButton  = this->win->searchForWidget("menuKeyButton");
	this->uplabel = dynamic_cast<MMSLabelWidget *>(this->win->searchForWidget("uplabel"));
	this->downlabel = dynamic_cast<MMSLabelWidget *>(this->win->searchForWidget("downlabel"));

	if(!lodgerMenu) {
		fprintf(stderr,"namesMenu not found in window!\n");
		exit(1);
	}

	unsigned int i = 0;
	for(std::map<string, User*>::iterator it = users->begin(); it != users->end(); it++) {
		lodger = lodgerMenu->newItem();
		if(lodger) {
			lodgername = dynamic_cast<MMSLabelWidget *>(lodger->searchForWidget("nameLabel"));
			if(lodgername) {
				lodgername->setText(it->second->name, true);
			}
			lodger->setData(it->second->sipid);
			if(i++ == 5) {
				checkbox = dynamic_cast<MMSCheckBoxWidget *>(lodger->findWidget("hline"));
				if(checkbox) {
					checkbox->setChecked(false);
				}
			}
		}
	}

	up = this->win->searchForWidget("menuUp");
	up->onReturn->connect(sigc::mem_fun(this, &MainMenu::onUp));
	up->setActivated(false);
	down = this->win->searchForWidget("menuDown");
	down->onReturn->connect(sigc::mem_fun(this, &MainMenu::onDown));

    // add animation callbacks
    this->onBeforeAnimation_connection = this->animThread.onBeforeAnimation.connect(sigc::mem_fun(this, &MainMenu::onBeforeAnimation));
    this->onAnimation_connection = this->animThread.onAnimation.connect(sigc::mem_fun(this, &MainMenu::onAnimation));
}

MainMenu::~MainMenu() {

}

void MainMenu::onUp(MMSWidget *widget) {
	if(lastAct == ACT_DOWN && lastSel > 5) {
		lastSel -= 5;
	}
	lodgerMenu->setSelected(lastSel);

	MMSWidget *item = lodgerMenu->getItem(lastSel + 3);
	MMSCheckBoxWidget *checkbox = dynamic_cast<MMSCheckBoxWidget *>(item->findWidget("hline"));
	if(checkbox) {
		checkbox->setChecked(true);
	}
	lodgerMenu->scrollUp();

	item = lodgerMenu->getItem(lastSel + 4);
	checkbox = dynamic_cast<MMSCheckBoxWidget *>(item->findWidget("hline"));
	if(checkbox) {
		checkbox->setChecked(false);
	}

	lastSel = lodgerMenu->getSelected();
	lastAct = ACT_UP;

	if(lastSel == 0) {
		up->setActivated(false);
		down->setActivated(true);
		downlabel->setActivated(true);
		uplabel->setActivated(false);

		uplabel->setColor(MMSFBColor(20,20,20,20));
		uplabel->setSelColor(MMSFBColor(20,20,20,200));
		downlabel->setColor(MMSFBColor(204,204,204,255));
		downlabel->setSelColor(MMSFBColor(204,204,204,255));

	} else {
		up->setActivated(true);
		down->setActivated(true);

		downlabel->setColor(MMSFBColor(204,204,204,255));
		downlabel->setSelColor(MMSFBColor(204,204,204,255));
		uplabel->setColor(MMSFBColor(204,204,204,255));
		uplabel->setSelColor(MMSFBColor(204,204,204,255));
	}
}

void MainMenu::onDown(MMSWidget *widget) {
	if(lastAct == ACT_UP && lastSel < 5) {
		lastSel += 5;
	}
	lodgerMenu->setSelected(lastSel);
	MMSCheckBoxWidget *checkbox;

	MMSWidget *item = lodgerMenu->getItem(lastSel + 1);
	if(item) {
		checkbox = dynamic_cast<MMSCheckBoxWidget *>(item->findWidget("hline"));
		if(checkbox) {
			checkbox->setChecked(false);
		}
	}

	item = lodgerMenu->getItem(lastSel);
	checkbox = dynamic_cast<MMSCheckBoxWidget *>(item->findWidget("hline"));
	if(checkbox) {
		checkbox->setChecked(true);
	}

	lodgerMenu->scrollDown();

	lastSel = lodgerMenu->getSelected();
	lastAct = ACT_DOWN;

	if(lastSel == users->size() - 1) {
		down->setActivated(false);
		up->setActivated(true);
		downlabel->setColor(MMSFBColor(20,20,20,20));
		downlabel->setSelColor(MMSFBColor(20,20,20,200));
		uplabel->setColor(MMSFBColor(204,204,204,255));
		uplabel->setSelColor(MMSFBColor(204,204,204,255));
	} else {
		down->setActivated(true);
		up->setActivated(true);

		downlabel->setColor(MMSFBColor(204,204,204,255));
		downlabel->setSelColor(MMSFBColor(204,204,204,255));
		uplabel->setColor(MMSFBColor(204,204,204,255));
		uplabel->setSelColor(MMSFBColor(204,204,204,255));
	}
}

void MainMenu::rewind() {

	if(lastSel==0  || lastSel == 5)
		return;

	if(!anim_saved_screen) {
		MMSFBLayer *layer=NULL;
		mmsfb->getLayer(0, &layer);

		if (layer) {
			MMSFBSurfacePixelFormat pixelformat;
			layer->getPixelFormat(&pixelformat);
			int w, h;
			layer->getResolution(&w, &h);
			layer->createWindow(&anim_saved_screen, 0, 0, w, h,
								pixelformat, isAlphaPixelFormat(pixelformat), 0);
		}
	}
	if (anim_saved_screen) {
		// get a screenshot
		anim_saved_screen->getScreenshot();

		// show the saved screen
		anim_saved_screen->raiseToTop();
		anim_saved_screen->setOpacity(255);
		anim_saved_screen->show();
	}

	lastSel = 1;
	onUp(up);

    if (anim_saved_screen) {
    	// do the animation
    	this->animThread.setStepsPerSecond(255);
    	this->animThread.start(false);

    	// delete the temporary window
    	this->anim_saved_screen->hide();
//    	delete this->anim_saved_screen;
//    	this->anim_saved_screen = NULL;
    }

	/*while(lodgerMenu->getSelected() > 0) {
		onUp(up);
	}*/
}

bool MainMenu::onBeforeAnimation(MMSPulser *animThread) {
	// init animation opacity
	this->anim_opacity = 255;

	return true;
}

bool MainMenu::onAnimation(MMSPulser *animThread) {
	// get new opacity
	this->anim_opacity-= animThread->getStepLength();

	// animation finished?
	if (this->anim_opacity <= 0) {
		// yes
		return false;
	}

	// set new opacity
	this->anim_saved_screen->setOpacity(this->anim_opacity);

	return true;
}
