/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#include <cstdlib>

#include "ringtoneThread.h"

#define RINGTONE_DELAY	1		/**< delay between ringtones in seconds */

RingtoneThread::RingtoneThread(const string ringtoneFile, const string busytoneFile, const string calltoneFile) :
	ringtoneFile(ringtoneFile),
	busytoneFile(busytoneFile),
	calltoneFile(calltoneFile),
	ringtoneCmd("aplay -q " + ringtoneFile),
	busytoneCmd("aplay -q -D phone " + busytoneFile),
	calltoneCmd("aplay -q -D phone " + calltoneFile),
	playCond(PLAY_NOTHING) {
	pthread_mutex_init(&this->lock, NULL);
	pthread_cond_init(&this->cond, NULL);
}

RingtoneThread::~RingtoneThread() {
	pthread_cond_destroy(&this->cond);
	pthread_mutex_destroy(&this->lock);
}

void RingtoneThread::threadMain() {
	struct timespec ts;

	while(true) {
		pthread_mutex_lock(&this->lock);
		clock_gettime(CLOCK_REALTIME, &ts);
		ts.tv_sec += RINGTONE_DELAY;
		pthread_cond_timedwait(&this->cond, &this->lock, &ts);
		switch(this->playCond) {
			case PLAY_RINGTONE:
				system(ringtoneCmd.c_str());
				break;
			case PLAY_BUSYTONE:
				system(busytoneCmd.c_str());
				break;
			case PLAY_CALLTONE:
				system(calltoneCmd.c_str());
				break;
			default:
				break;
		}
		pthread_mutex_unlock(&this->lock);
	}
}

void RingtoneThread::startRingtone() {
	pthread_mutex_lock(&this->lock);
	if(!this->ringtoneFile.empty()) {
		this->playCond = PLAY_RINGTONE;
		pthread_cond_signal(&this->cond);
	}
	pthread_mutex_unlock(&this->lock);
}

void RingtoneThread::startBusytone() {
	pthread_mutex_lock(&this->lock);
	if(!this->busytoneFile.empty()) {
		this->playCond = PLAY_BUSYTONE;
		pthread_cond_signal(&this->cond);
	}
	pthread_mutex_unlock(&this->lock);
}

void RingtoneThread::startCalltone() {
	pthread_mutex_lock(&this->lock);
	if(!this->calltoneFile.empty()) {
		this->playCond = PLAY_CALLTONE;
		pthread_cond_signal(&this->cond);
	}
	pthread_mutex_unlock(&this->lock);
}

void RingtoneThread::stopTone() {
	pthread_mutex_lock(&this->lock);
	this->playCond = PLAY_NOTHING;
	pthread_cond_signal(&this->cond);
	pthread_mutex_unlock(&this->lock);
}
