/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#include "themeswitchthread.h"
#include "sunphase.h"


ThemeSwitchThread::ThemeSwitchThread(string &currtheme) : sleepcount(0), currtheme(""){
	this->currtheme = currtheme;
	MMSConfigData config;
	DataSource datasource(config.getConfigDBDBMS(),
						  config.getConfigDBDatabase(),
						  config.getConfigDBAddress(),
						  config.getConfigDBPort(),
						  config.getConfigDBUser(),
						  config.getConfigDBPassword());

	MMSPluginService service(&datasource);
	MMSPROPERTYDATA_MAP props;
	service.getSystemProperties(props);

	MMSPROPERTYDATA_MAP::iterator longi = props.find("Longitude");
	MMSPROPERTYDATA_MAP::iterator lati = props.find("Latitude");

	this->longitude = strtod(longi->second->getValue().c_str(), NULL);
	this->latitude = strtod(lati->second->getValue().c_str(), NULL);
}

ThemeSwitchThread::~ThemeSwitchThread() {

}

void ThemeSwitchThread::threadMain() {
	time_t now, set, rise;
	MMSThemeManager tm;

	while(1) {
		now = time(NULL);
		calcSunPhase(rise,set,now,this->longitude,this->latitude);

		if(currtheme == "night") {
			/* were after sunset */
			if(set < now) {
				calcSunPhase(rise,set,now+24*3600,this->longitude,this->latitude);
				printf("sleep %ld seconds until sunrise tomorrow\n", rise-now);
				sleep(rise-now);
				currtheme="day";
				tm.setTheme(currtheme);

			} else if(now < rise) {
				//were before sunset
				printf("sleep %ld seconds until sunrise\n", rise-now);
				sleep(rise-now);
				currtheme="day";
				tm.setTheme(currtheme);
			}
		} else if(currtheme == "day") {
			//were in the day
			printf("sleep %ld seconds until sunset\n", set-now);
			sleep(set-now);
			currtheme="night";
			tm.setTheme(currtheme);
		}
		sleep(1800);
	}

}

