/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#include "userfilereader.h"
#include <cerrno>
#include <cstring>
#include <cstdlib>
#include <cstdio>

UserFileReader::UserFileReader(string userFileUri) {
	uri = userFileUri;
}

UserFileReader::~UserFileReader() {
	//clearUsers();
}

bool UserFileReader::read() {
	MMSFile userFile(uri, MMSFM_READ, false);
	string line;
	char *lineBuffer;
	char *lineBufferPtr;
	char *tok;
	int cnt;

	clearUsers();

	User *u;

	while(userFile.getLine(line)) {
		cnt = 0;
		u = new User;
		u->counter = 0;
		lineBuffer = strdup(line.c_str());
		lineBufferPtr = lineBuffer;

		while((tok = strtok(lineBuffer, ";"))) {
			if(cnt < 3 && strlen(tok) == 0) {
				clearUsers();
				delete u;
				return false;
			}
			switch(cnt) {
				case 0:
					u->sipid.append(tok);
					break;
				case 1:
					u->name.append(tok);
					break;
				case 2:
					u->code.append(tok);
					break;
				case 3:
					u->awayMsg.append(tok);
					break;
			}
			++cnt;
			lineBuffer = 0;
		}

		users.push_back(u);
		free(lineBufferPtr);
	}

	int err = userFile.getLastError();
	if(err > 0) {
		printf("error: %d - %s\n", err, strerror(err));
		return false;
	}

	return true;
}

vector<User *> UserFileReader::getUsers() {
	return users;
}

void UserFileReader::clearUsers() {
	for(size_t i = 0; i < users.size(); ++i) {
		delete users[i];
	}

	users.clear();
}
