/***************************************************************************
 * This file is part of the BF527 doorbell application.                    *
 * Doorbell is a VOIP-based outdoor doorbell that can be run on SIP-based  *
 * telephone systems (PBX).                                                *
 *                                                                         *
 * Copyright (C) 2009,2010 by BerLinux Solutions GmbH                      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#include "volumedialog.h"

unsigned int volumeValues[4] = {70, 80, 90, 100};

VolumeDialog::VolumeDialog(Sipbackend *sip) :
		Dialog::Dialog(false),
		volumeMenu(NULL),
		volumeImg(NULL),
		upArrow(NULL),
		upLabel(NULL),
		downArrow(NULL),
		downLabel(NULL),
		sipBackend(sip) {
	loadDialog("volume.xml");

	this->win->onBeforeShow->connect(
			sigc::mem_fun(this, &VolumeDialog::onBeforeShow));

	this->volumeMenu = dynamic_cast<MMSMenuWidget *>(
			this->win->searchForWidget("volumeBar"));
	if(this->volumeMenu) {
		this->volumeImg = dynamic_cast<MMSImageWidget *>(
				this->win->searchForWidget("volumeImage"));

		this->upArrow = this->win->searchForWidget("volUpArrow");
		if(this->upArrow) {
			this->upLabel = dynamic_cast<MMSLabelWidget *>(this->upArrow->searchForWidget("label"));
		}

		this->downArrow = this->win->searchForWidget("volDownArrow");
		if(this->downArrow) {
			this->downLabel = dynamic_cast<MMSLabelWidget *>(this->downArrow->searchForWidget("label"));
		}

		this->volumeMenu->onSelectItem->connect(
				sigc::mem_fun(this, &VolumeDialog::onSelectItem));

		volumeImg0 = new MMSImageWidget(this->win, "",NULL);
		volumeImg0->setImageName("icon_vol_0.png", true, true);

		volumeImg1 = new MMSImageWidget(this->win, "",NULL);
		volumeImg1->setImageName("icon_vol_1.png", true, true);

		volumeImg2 = new MMSImageWidget(this->win, "",NULL);
		volumeImg2->setImageName("icon_vol_3.png", true, true);

		volumeImg3 = new MMSImageWidget(this->win, "",NULL);
		volumeImg3->setImageName("icon_vol_3.png", true, true);
	}
}

VolumeDialog::~VolumeDialog() {

}

bool VolumeDialog::onBeforeShow(MMSWindow *win) {
	if(this->volumeMenu) {
		bool changed;
		this->volumeMenu->setSelected(3, false, &changed);
		if(!changed && sipBackend) {
			sipBackend->setVolume(volumeValues[3]);
		}
	}

	return true;
}

void VolumeDialog::onSelectItem(MMSWidget *wid)
{
	static unsigned int oldSelected = 3;
	
	unsigned int selected = this->volumeMenu->getSelected();
	switch(selected) {
		case 0:
			if(this->downArrow)	{
				this->downArrow->setActivated(false, false);
				if(this->downLabel) {
					this->downLabel->setColor(MMSFBColor(20,20,20,20), false);
					this->downLabel->setSelColor(MMSFBColor(20,20,20,20), false);
				}
			}
			break;
		case 1:
			if((oldSelected == 0) && this->downArrow)	{
				this->downArrow->setActivated(true, false);
				if(this->downLabel) {
					this->downLabel->setColor(MMSFBColor(204,204,204,255), false);
					this->downLabel->setSelColor(MMSFBColor(204,204,204,255), false);
				}
			}
			break;
		case 2:
			if((oldSelected == 3) && this->upArrow) {
				this->upArrow->setActivated(true, false);
				if(this->upLabel) {
					this->upLabel->setColor(MMSFBColor(204,204,204,255), false);
					this->upLabel->setSelColor(MMSFBColor(204,204,204,255), false);
				}
			}
			break;
		case 3:
			if(this->upArrow) {
				this->upArrow->setActivated(false, false);
				if(this->upLabel) {
					this->upLabel->setColor(MMSFBColor(20,20,20,20), false);
					this->upLabel->setSelColor(MMSFBColor(20,20,20,20), false);
				}
			}
			break;
	}
	oldSelected = selected;
	
	if(this->volumeImg) {
		this->volumeImg->setImageName("icon_vol_" + iToStr(selected) + ".png", true, false);
	}

	this->win->refresh();

	if(sipBackend) {
		sipBackend->setVolume(volumeValues[selected]);
	}
}
